using System;
using System.Collections.Generic;
using System.Text;
using System.Data.OleDb;
using Microsoft.SqlServer.Dts.Pipeline;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;
using System.Collections;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Runtime.Wrapper;

namespace DataFlowComponent
{
    [DtsPipelineComponent(DisplayName = "Capitalize Column Component",
    ComponentType = ComponentType.Transform,
    Description = "Capitalizes the specified Column",
    RequiredProductLevel = Microsoft.SqlServer.Dts.Runtime.Wrapper.DTSProductLevel.DTSPL_NONE,
    UITypeName = "DataFlowComponent.CapitalizeUI,DataFlowComponent,Version=1.0.0.0,Culture=neutral,PublicKeyToken=6cc73905e2483584",
    IconResource = "DataFlowComponent.Capitalize.ico")]
    public class Capitalize : PipelineComponent
    {
        #region Instance Variables
        #region Constants
        const string CapitalizeColumn = "CapitalizeColumn";
        #endregion
        #region Dictionaries
        private Dictionary<string, int> columnIndexes = new Dictionary<string,int>();
        #endregion

        private IDTSOutput90 output;
        private IDTSInput90 input;

        #endregion

        #region Overrides

        #region Component Methods

        public override DTSValidationStatus Validate()
        {
            if (ComponentMetaData.InputCollection.Count != 1 || ComponentMetaData.OutputCollection.Count != 1)
            {
                return DTSValidationStatus.VS_NEEDSNEWMETADATA;
            }
            return base.Validate();
        }

        #endregion

        #region Design Time Methods

        public override void ProvideComponentProperties()
        {
            base.ProvideComponentProperties();

            //Support Resetting of component
            this.RemoveAllInputsOutputsAndCustomProperties();
            this.ComponentMetaData.ValidateExternalMetadata = true;

            //Custom Properties - Column Name to store row number in
            IDTSCustomProperty90 customPropColumnName = ComponentMetaData.CustomPropertyCollection.New();
            customPropColumnName.Name = CapitalizeColumn;
            customPropColumnName.Value = "";
            customPropColumnName.ExpressionType = DTSCustomPropertyExpressionType.CPET_NOTIFY;

            //Add Input Collection 
            input = ComponentMetaData.InputCollection.New();
            input.Name = "Input";

            //Add Output Collection
            output = ComponentMetaData.OutputCollection.New();
            output.Name = "Output";

            //Synchronize Input to output
            output.SynchronousInputID = input.ID;
        }

        public override void ReinitializeMetaData()
        {
            this.RemoveAllInputsOutputsAndCustomProperties();
            //Add Input Collection 
            input = ComponentMetaData.InputCollection.New();
            input.Name = "Input";

            //Add Output Collection
            output = ComponentMetaData.OutputCollection.New();
            //Synchronize Input to output
            output.SynchronousInputID = input.ID;
            base.ReinitializeMetaData();
        }

        public override void OnInputPathAttached(int inputID)
        {
            //Populate the Custom Property with the value of the first column when input path is attached
            IDTSVirtualInput90 input = ComponentMetaData.InputCollection[0].GetVirtualInput();
            ComponentMetaData.CustomPropertyCollection[CapitalizeColumn].Value = input.VirtualInputColumnCollection[0].Name;
            base.OnInputPathAttached(inputID);
        }
        #endregion

        #region Execution Methods

        public override void PreExecute()
        {
            bool pbCancel = false;
            try
            {
                output = ComponentMetaData.OutputCollection[0];
                input = ComponentMetaData.InputCollection[0];
                IDTSVirtualInput90 vIn = input.GetVirtualInput();
                int i =0;
                for(i = 0; i < vIn.VirtualInputColumnCollection.Count; i++)
                {
                    IDTSVirtualInputColumn90 outCol = vIn.VirtualInputColumnCollection[i];
                    columnIndexes.Add(outCol.Name, i);
                }
            }
            catch (Exception ex)
            {
                ComponentMetaData.FireError(06, ComponentMetaData.Name, "Error occurred on Pre-Execute in " + ComponentMetaData.Name + ". " + ex.Message, "", 0, out pbCancel);
            }
            base.PreExecute();
        }

        public override void PrimeOutput(int outputs, int[] outputIDs, PipelineBuffer[] buffers)
        {
            base.PrimeOutput(outputs, outputIDs, buffers);
        }

        public override void  ProcessInput(int inputID, PipelineBuffer buffer)
        {
            try
            {
                while (buffer.NextRow())
                {

                    buffer[columnIndexes[ComponentMetaData.CustomPropertyCollection[CapitalizeColumn].Value.ToString()]] = 
                        buffer[columnIndexes[ComponentMetaData.CustomPropertyCollection[CapitalizeColumn].Value.ToString()]].ToString().ToUpper();
                }
            }
            catch (Exception ex)
            {

            }
 	        base.ProcessInput(inputID, buffer);
        }

        public override void PostExecute()
        {
            base.PostExecute();
        }
        #endregion
        #endregion
    }
}
